@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA.
  *
  * The Totem project hereby grant permission for non-gpl compatible GStreamer
  * plugins to be used and distributed together with GStreamer and Totem. This
@@ -61,7 +61,6 @@ typedef struct
 G_MODULE_EXPORT GType register_totem_plugin		(GTypeModule *module);
 GType	totem_movie_properties_plugin_get_type		(void) G_GNUC_CONST;
 
-static void totem_movie_properties_plugin_init		(TotemMoviePropertiesPlugin *plugin);
 static gboolean impl_activate				(TotemPlugin *plugin, TotemObject *totem, GError **error);
 static void impl_deactivate				(TotemPlugin *plugin, TotemObject *totem);
 
@@ -126,21 +125,15 @@ totem_movie_properties_plugin_metadata_updated (TotemObject *totem,
 						const char *artist, 
 						const char *title, 
 						const char *album,
+						guint track_num,
 						TotemMoviePropertiesPlugin *plugin)
 {
-	if (artist == NULL && title == NULL && album == NULL) {
-		GtkWidget *bvw;
-
-		bvw = totem_get_video_widget (totem);
-		bacon_video_widget_properties_update
-			(BACON_VIDEO_WIDGET_PROPERTIES (plugin->props), bvw);
-		g_object_unref (bvw);
-	} else {
-		bacon_video_widget_properties_from_metadata
-			(BACON_VIDEO_WIDGET_PROPERTIES (plugin->props),
-			 artist, title, album);
-	}
+	GtkWidget *bvw;
 
+	bvw = totem_get_video_widget (totem);
+	bacon_video_widget_properties_update
+		(BACON_VIDEO_WIDGET_PROPERTIES (plugin->props), bvw);
+	g_object_unref (bvw);
 }
 
 static gboolean
@@ -189,6 +182,15 @@ impl_deactivate	(TotemPlugin *plugin,
 	pi = TOTEM_MOVIE_PROPERTIES_PLUGIN (plugin);
 
 	g_signal_handler_disconnect (G_OBJECT (totem), pi->handler_id_stream_length);
+	g_signal_handlers_disconnect_by_func (G_OBJECT (totem),
+					      totem_movie_properties_plugin_metadata_updated,
+					      plugin);
+	g_signal_handlers_disconnect_by_func (G_OBJECT (totem),
+					      totem_movie_properties_plugin_file_opened,
+					      plugin);
+	g_signal_handlers_disconnect_by_func (G_OBJECT (totem),
+					      totem_movie_properties_plugin_file_closed,
+					      plugin);
 	pi->handler_id_stream_length = 0;
 	totem_remove_sidebar_page (totem, "properties");
 }
