@@ -31,11 +31,12 @@
 #include "totem-interface.h"
 #include "totem-private.h"
 #include "totem-sidebar.h"
+#include "totem-statusbar.h"
 #include "totem-plugin-manager.h"
 #include "bacon-video-widget.h"
 #include "totem-uri.h"
 
-#include "debug.h"
+#include "totem-profile.h"
 
 #define TOTEM_MAX_RECENT_ITEM_LEN 40
 
@@ -523,7 +524,7 @@ connect_proxy_cb (GtkActionGroup *action_group,
         if (!GTK_IS_MENU_ITEM (proxy))
                 return;
 
-        label = GTK_LABEL (GTK_BIN (proxy)->child);
+        label = GTK_LABEL (gtk_bin_get_child (GTK_BIN (proxy)));
 
         gtk_label_set_ellipsize (label, PANGO_ELLIPSIZE_MIDDLE);
         gtk_label_set_max_width_chars (label,TOTEM_MAX_RECENT_ITEM_LEN);
@@ -543,6 +544,7 @@ connect_proxy_cb (GtkActionGroup *action_group,
         }
 
         gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (proxy), image);
+        gtk_image_menu_item_set_always_show_image (GTK_IMAGE_MENU_ITEM (proxy), TRUE);
 }
 
 static void
@@ -686,51 +688,65 @@ totem_setup_recent (Totem *totem)
 	totem_recent_manager_changed_callback (totem->recent_manager, totem);
 }
 
-void
-totem_action_add_recent (Totem *totem, const char *uri)
+static void
+recent_info_cb (GFile *file,
+		GAsyncResult *res,
+		Totem *totem)
 {
 	GtkRecentData data;
 	char *groups[] = { NULL, NULL };
-	GFile *file;
 	GFileInfo *file_info;
+	const char *uri, *display_name;
 
 	memset (&data, 0, sizeof (data));
 
-	if (totem_is_special_mrl (uri) != FALSE)
-		return;
-
-	file = g_file_new_for_uri (uri);
-	file_info = g_file_query_info (file,
-				       G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE "," G_FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME,
-				       G_FILE_QUERY_INFO_NONE, NULL, NULL);
+	file_info = g_file_query_info_finish (file, res, NULL);
+	uri = g_object_get_data (G_OBJECT (file), "uri");
+	display_name = g_object_get_data (G_OBJECT (file), "display_name");
 
 	/* Probably an unsupported URI scheme */
 	if (file_info == NULL) {
-		data.display_name = NULL;
+		data.display_name = g_strdup (display_name);
 		/* Bogus mime-type, we just want it added */
 		data.mime_type = g_strdup ("video/x-totem-stream");
 		groups[0] = (gchar*) "TotemStreams";
-		g_message ("no file info");
 	} else {
 		data.mime_type = g_strdup (g_file_info_get_content_type (file_info));
 		data.display_name = g_strdup (g_file_info_get_display_name (file_info));
 		g_object_unref (file_info);
 		groups[0] = (gchar*) "Totem";
 	}
-	g_object_unref (file);
 
 	data.app_name = g_strdup (g_get_application_name ());
 	data.app_exec = g_strjoin (" ", g_get_prgname (), "%u", NULL);
 	data.groups = groups;
 	if (gtk_recent_manager_add_full (totem->recent_manager,
-				     uri, &data) == FALSE) {
-		g_message ("Couldn't add recent file for '%s'", uri);
+					 uri, &data) == FALSE) {
+		g_warning ("Couldn't add recent file for '%s'", uri);
 	}
 
 	g_free (data.display_name);
 	g_free (data.mime_type);
 	g_free (data.app_name);
 	g_free (data.app_exec);
+
+	g_object_unref (file);
+}
+
+void
+totem_action_add_recent (Totem *totem, const char *uri, const char *display_name)
+{
+	GFile *file;
+
+	if (totem_is_special_mrl (uri) != FALSE)
+		return;
+
+	file = g_file_new_for_uri (uri);
+	g_object_set_data_full (G_OBJECT (file), "uri", g_strdup (uri), g_free);
+	g_object_set_data_full (G_OBJECT (file), "display_name", g_strdup (display_name), g_free);
+	g_file_query_info_async (file,
+				 G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE "," G_FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME,
+				 G_FILE_QUERY_INFO_NONE, 0, NULL, (GAsyncReadyCallback) recent_info_cb, totem);
 }
 
 /* Play Disc menu items */
@@ -744,69 +760,24 @@ on_play_disc_activate (GtkAction *action, Totem *totem)
 	totem_action_play_media_device (totem, device_path);
 }
 
-/* Play DVB menu items */
-static void
-on_play_dvb_activate (GtkAction *action, Totem *totem)
+static const char *
+get_icon_name_for_gicon (GtkIconTheme *theme,
+			 GIcon *icon)
 {
-	int adapter;
-	char *str;
+	const char * const *icon_names;
+	const char *icon_name;
+	guint j;
 
-	adapter = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (action), "adapter"));
-	str = g_strdup_printf ("%d", adapter);
-	totem_action_play_media (totem, MEDIA_TYPE_DVB, str);
-	g_free (str);
-}
+	icon_name = NULL;
 
-static void
-add_drive_to_menu (GDrive *drive, guint position, Totem *totem)
-{
-	GtkIconTheme *theme;
-	GList *volumes, *i;
+	if (G_IS_EMBLEMED_ICON (icon) != FALSE) {
+		GIcon *new_icon;
+		new_icon = g_emblemed_icon_get_icon (G_EMBLEMED_ICON (icon));
+		g_object_unref (icon);
+		icon = g_object_ref (new_icon);
+	}
 
-	theme = gtk_icon_theme_get_default ();
-
-	/* Repeat for all the drive's volumes */
-	volumes = g_drive_get_volumes (drive);
-
-	for (i = volumes; i != NULL; i = i->next) {
-		char *name, *escaped_name, *label;
-		GtkAction *action;
-		gboolean disabled;
-		GIcon *icon;
-		const char * const *icon_names;
-		const char *icon_name;
-		guint j;
-		char *device_path;
-
-		disabled = FALSE;
-
-		/* Add devices with blank CDs and audio CDs in them, but disable them */
-		device_path = g_volume_get_identifier (i->data, G_VOLUME_IDENTIFIER_KIND_UNIX_DEVICE);
-		if (device_path == NULL)
-			continue;
-
-		/* Check whether we have a media... */
-		if (g_drive_has_media (drive) == FALSE) {
-			disabled = TRUE;
-		} else {
-			/* ... Or an audio CD or a blank media */
-			GMount *mount;
-			GFile *root;
-
-			mount = g_volume_get_mount (i->data);
-			if (mount != NULL) {
-				root = g_mount_get_root (mount);
-				g_object_unref (mount);
-
-				if (g_file_has_uri_scheme (root, "burn") != FALSE || g_file_has_uri_scheme (root, "cdda") != FALSE)
-					disabled = TRUE;
-				g_object_unref (root);
-			}
-		}
-
-		/* Work out an icon to display */
-		icon = g_volume_get_icon (i->data);
-		icon_name = NULL;
+	if (G_IS_THEMED_ICON (icon)) {
 		icon_names = g_themed_icon_get_names (G_THEMED_ICON (icon));
 
 		for (j = 0; icon_names[j] != NULL; j++) {
@@ -814,43 +785,256 @@ add_drive_to_menu (GDrive *drive, guint position, Totem *totem)
 			if (gtk_icon_theme_has_icon (theme, icon_name) != FALSE)
 				break;
 		}
+	}
 
-		/* Get the volume's pretty name for the menu label */
-		name = g_volume_get_name (i->data);
-		g_strstrip (name);
-		escaped_name = escape_label_for_menu (name);
-		g_free (name);
-		label = g_strdup_printf (_("Play Disc '%s'"), escaped_name);
-		g_free (escaped_name);
+	return icon_name;
+}
 
-		name = g_strdup_printf (_("device%d"), position);
+static char *
+unescape_archive_name (GFile *root)
+{
+	char *uri;
+	guint len;
+	char *escape1, *escape2;
 
-		action = gtk_action_new (name, label, NULL, NULL);
-		g_object_set (G_OBJECT (action),
-			      "icon-name", icon_name,
-			      "sensitive", !disabled, NULL);
-		gtk_action_group_add_action (totem->devices_action_group, action);
-		g_object_unref (action);
+	uri = g_file_get_uri (root);
 
-		gtk_ui_manager_add_ui (totem->ui_manager, totem->devices_ui_id,
-			"/tmw-menubar/movie/devices-placeholder", name, name,
-			GTK_UI_MANAGER_MENUITEM, FALSE);
+	/* Remove trailing slash */
+	len = strlen (uri);
+	if (uri[len - 1] == '/')
+		uri[len - 1] = '\0';
 
-		g_free (name);
-		g_free (label);
-		g_object_unref (icon);
+	/* Unescape the path */
+	escape1 = g_uri_unescape_string (uri + strlen ("archive://"), NULL);
+	escape2 = g_uri_unescape_string (escape1, NULL);
+	g_free (escape1);
+	g_free (uri);
 
-		if (disabled != FALSE) {
-			g_free (device_path);
-			return;
+	return escape2;
+}
+
+static void
+add_mount_to_menu (GMount *mount,
+		   GtkIconTheme *theme,
+		   guint position,
+		   Totem *totem)
+{
+	char *name, *escaped_name, *label;
+	GtkAction *action;
+	GIcon *icon;
+	const char *icon_name;
+	char *device_path;
+
+	GVolume *volume;
+	GFile *root, *iso;
+	char **content_types;
+	gboolean has_content;
+	guint i;
+
+	/* Check whether we have an archive mount */
+	volume = g_mount_get_volume (mount);
+	if (volume != NULL) {
+		g_object_unref (volume);
+		return;
+	}
+
+	root = g_mount_get_root (mount);
+	if (g_file_has_uri_scheme (root, "archive") == FALSE) {
+		g_object_unref (root);
+		return;
+	}
+
+	/* Check whether it's a DVD or VCD image */
+	content_types = g_content_type_guess_for_tree (root);
+	if (content_types == NULL ||
+	    g_strv_length (content_types) == 0) {
+		g_strfreev (content_types);
+		g_object_unref (root);
+		return;
+	}
+
+	has_content = FALSE;
+	for (i = 0; content_types[i] != NULL; i++) {
+		/* XXX: Keep in sync with mime-type-list.txt */
+		if (g_str_equal (content_types[i], "x-content/video-dvd") ||
+		    g_str_equal (content_types[i], "x-content/video-vcd") ||
+		    g_str_equal (content_types[i], "x-content/video-svcd")) {
+			has_content = TRUE;
+			break;
 		}
+	}
+	g_strfreev (content_types);
 
-		g_object_set_data_full (G_OBJECT (action),
-					"device_path", device_path,
-					(GDestroyNotify) g_free);
+	if (has_content == FALSE) {
+		g_object_unref (root);
+		return;
+	}
 
-		g_signal_connect (G_OBJECT (action), "activate",
-				  G_CALLBACK (on_play_disc_activate), totem);
+	device_path = unescape_archive_name (root);
+	g_object_unref (root);
+
+	/* And ensure it's a local path */
+	iso = g_file_new_for_uri (device_path);
+	g_free (device_path);
+	device_path = g_file_get_path (iso);
+	g_object_unref (iso);
+
+	/* Work out an icon to display */
+	icon = g_mount_get_icon (mount);
+	icon_name = get_icon_name_for_gicon (theme, icon);
+
+	/* Get the mount's pretty name for the menu label */
+	name = g_mount_get_name (mount);
+	g_strstrip (name);
+	escaped_name = escape_label_for_menu (name);
+	g_free (name);
+	/* Translators:
+	 * This is not a JPEG image, but a disc image, for example,
+	 * an ISO file */
+	label = g_strdup_printf (_("Play Image '%s'"), escaped_name);
+	g_free (escaped_name);
+
+	name = g_strdup_printf (_("device%d"), position);
+
+	action = gtk_action_new (name, label, NULL, NULL);
+	g_object_set (G_OBJECT (action),
+		      "icon-name", icon_name, NULL);
+	gtk_action_set_always_show_image (action, TRUE);
+	gtk_action_group_add_action (totem->devices_action_group, action);
+	g_object_unref (action);
+
+	gtk_ui_manager_add_ui (totem->ui_manager, totem->devices_ui_id,
+			       "/tmw-menubar/movie/devices-placeholder", name, name,
+			       GTK_UI_MANAGER_MENUITEM, FALSE);
+
+	g_free (name);
+	g_free (label);
+	g_object_unref (icon);
+
+	g_object_set_data_full (G_OBJECT (action),
+				"device_path", device_path,
+				(GDestroyNotify) g_free);
+
+	g_signal_connect (G_OBJECT (action), "activate",
+			  G_CALLBACK (on_play_disc_activate), totem);
+}
+
+static void
+add_volume_to_menu (GVolume *volume,
+		    GDrive *drive,
+		    GtkIconTheme *theme,
+		    guint position,
+		    Totem *totem)
+{
+	char *name, *escaped_name, *label;
+	GtkAction *action;
+	gboolean disabled;
+	GIcon *icon;
+	const char *icon_name;
+	char *device_path;
+	GtkWidget *menu_item;
+	char *menu_item_path;
+
+	disabled = FALSE;
+
+	/* Add devices with blank CDs and audio CDs in them, but disable them */
+	if (drive != NULL) {
+		device_path = g_volume_get_identifier (volume, G_VOLUME_IDENTIFIER_KIND_UNIX_DEVICE);
+		if (device_path == NULL)
+			return;
+	}
+
+	/* Check whether we have a media... */
+	if (drive != NULL &&
+	    g_drive_has_media (drive) == FALSE) {
+		disabled = TRUE;
+	} else {
+		/* ... Or an audio CD or a blank media */
+		GMount *mount;
+		GFile *root;
+
+		mount = g_volume_get_mount (volume);
+		if (mount != NULL) {
+			root = g_mount_get_root (mount);
+			g_object_unref (mount);
+
+			if (g_file_has_uri_scheme (root, "burn") != FALSE || g_file_has_uri_scheme (root, "cdda") != FALSE)
+				disabled = TRUE;
+			g_object_unref (root);
+		}
+	}
+
+	/* Work out an icon to display */
+	icon = g_volume_get_icon (volume);
+	icon_name = get_icon_name_for_gicon (theme, icon);
+
+	/* Get the volume's pretty name for the menu label */
+	name = g_volume_get_name (volume);
+	g_strstrip (name);
+	escaped_name = escape_label_for_menu (name);
+	g_free (name);
+	label = g_strdup_printf (_("Play Disc '%s'"), escaped_name);
+	g_free (escaped_name);
+
+	name = g_strdup_printf (_("device%d"), position);
+
+	action = gtk_action_new (name, label, NULL, NULL);
+	g_object_set (G_OBJECT (action),
+		      "icon-name", icon_name,
+		      "sensitive", !disabled, NULL);
+	gtk_action_group_add_action (totem->devices_action_group, action);
+	g_object_unref (action);
+
+	gtk_ui_manager_add_ui (totem->ui_manager, totem->devices_ui_id,
+			       "/tmw-menubar/movie/devices-placeholder", name, name,
+			       GTK_UI_MANAGER_MENUITEM, FALSE);
+
+	/* TODO: This can be made cleaner once bug #589842 is fixed */
+	menu_item_path = g_strdup_printf ("/tmw-menubar/movie/devices-placeholder/%s", name);
+	menu_item = gtk_ui_manager_get_widget (totem->ui_manager, menu_item_path);
+	g_free (menu_item_path);
+
+	if (menu_item != NULL)
+		gtk_image_menu_item_set_always_show_image (GTK_IMAGE_MENU_ITEM (menu_item), TRUE);
+
+	g_free (name);
+	g_free (label);
+	g_object_unref (icon);
+
+	if (disabled != FALSE) {
+		g_free (device_path);
+		return;
+	}
+
+	g_object_set_data_full (G_OBJECT (action),
+				"device_path", device_path,
+				(GDestroyNotify) g_free);
+
+	g_signal_connect (G_OBJECT (action), "activate",
+			  G_CALLBACK (on_play_disc_activate), totem);
+}
+
+static void
+add_drive_to_menu (GDrive *drive,
+		   GtkIconTheme *theme,
+		   guint position,
+		   Totem *totem)
+{
+	GList *volumes, *i;
+
+	/* FIXME: We used to explicitly check whether it was a CD/DVD drive
+	 * Use:
+	 * udi = g_volume_get_identifier (i->data, G_VOLUME_IDENTIFIER_KIND_HAL_UDI); */
+	if (g_drive_can_eject (drive) == FALSE)
+		return;
+
+	/* Repeat for all the drive's volumes */
+	volumes = g_drive_get_volumes (drive);
+
+	for (i = volumes; i != NULL; i = i->next) {
+		GVolume *volume = i->data;
+		add_volume_to_menu (volume, drive, theme, position, totem);
+		g_object_unref (volume);
 	}
 
 	g_list_free (volumes);
@@ -859,76 +1043,45 @@ add_drive_to_menu (GDrive *drive, guint position, Totem *totem)
 static void
 update_drive_menu_items (GtkMenuItem *movie_menuitem, Totem *totem)
 {
-	GList *drives, *i;
+	GList *drives, *mounts, *i;
+	GtkIconTheme *theme;
 	guint position;
 
 	/* Add any suitable devices to the menu */
 	position = 0;
 
+	theme = gtk_icon_theme_get_default ();
+
 	drives = g_volume_monitor_get_connected_drives (totem->monitor);
 	for (i = drives; i != NULL; i = i->next) {
-		/* FIXME: We used to explicitly check whether it was a CD/DVD drive
-		 * Use:
-		 * udi = g_volume_get_identifier (i->data, G_VOLUME_IDENTIFIER_KIND_HAL_UDI); */
-		if (g_drive_can_eject (i->data) == FALSE)
-			continue;
+		GDrive *drive = i->data;
 
 		position++;
-		add_drive_to_menu (i->data, position, totem);
+		add_drive_to_menu (drive, theme, position, totem);
+		g_object_unref (drive);
 	}
 	g_list_free (drives);
 
+	/* Look for mounted archives */
+	mounts = g_volume_monitor_get_mounts (totem->monitor);
+	for (i = mounts; i != NULL; i = i->next) {
+		GMount *mount = i->data;
+
+		position++;
+		add_mount_to_menu (mount, theme, position, totem);
+		g_object_unref (mount);
+	}
+	g_list_free (mounts);
+
 	totem->drives_changed = FALSE;
 }
 
-static void
-update_dvb_menu_items (GtkMenuItem *movie_menuitem, Totem *totem)
-{
-	guint i;
-
-	for (i = 0 ; i < 8 ; i++) {
-		char *devicenode;
-
-		devicenode = g_strdup_printf("/dev/dvb/adapter%d/frontend0", i);
-
-		if (g_file_test (devicenode, G_FILE_TEST_EXISTS) != FALSE) {
-			char* label, *name, *adapter_name;
-			GtkAction* action;
-
-			/* translators: the index of the adapter
-			 * DVB Adapter 1 */
-			adapter_name = g_strdup_printf (_("DVB Adapter %u"), i);
-			/* translators:
-			 * Watch TV on 'DVB Adapter 1'
-			 * or
-			 * Watch TV on 'Hauppauge Nova-T Stick' */
-			label = g_strdup_printf (_("Watch TV on \'%s\'"), adapter_name);
-			g_free (adapter_name);
-			name = g_strdup_printf ("dvbdevice%d", i);
-			action = gtk_action_new (name, label, NULL, NULL);
-
-			g_object_set (G_OBJECT(action), "icon-name", "totem-tv", "sensitive", TRUE, NULL);
-			gtk_action_group_add_action (totem->devices_action_group, action);
-			g_object_unref (action);
-			gtk_ui_manager_add_ui (totem->ui_manager, totem->devices_ui_id,
-					       "/tmw-menubar/movie/devices-placeholder", name, name,
-					       GTK_UI_MANAGER_MENUITEM, FALSE);
-			g_object_set_data_full (G_OBJECT (action),
-						"adapter", GINT_TO_POINTER (i), NULL);
-			g_signal_connect (G_OBJECT (action), "activate",
-					  G_CALLBACK (on_play_dvb_activate), totem);
-		}
-		g_free (devicenode);
-	}
-}
-
 static void
 on_movie_menu_select (GtkMenuItem *movie_menuitem, Totem *totem)
 {
-	//FIXME we should check whether there's new DVB items
-/*	if (totem->drives_changed == FALSE)
+	if (totem->drives_changed == FALSE)
 		return;
-*/
+
 	/* Remove old UI */
 	gtk_ui_manager_remove_ui (totem->ui_manager, totem->devices_ui_id);
 	gtk_ui_manager_ensure_update (totem->ui_manager);
@@ -945,10 +1098,6 @@ on_movie_menu_select (GtkMenuItem *movie_menuitem, Totem *totem)
 
 	update_drive_menu_items (movie_menuitem, totem);
 
-	/* Check for DVB */
-	/* FIXME we should only update if we have an updated as per HAL */
-	update_dvb_menu_items (movie_menuitem, totem);
-
 	gtk_ui_manager_ensure_update (totem->ui_manager);
 }
 
@@ -1230,7 +1379,7 @@ plugins_action_callback (GtkAction *action, Totem *totem)
 							      GTK_RESPONSE_CLOSE,
 							      NULL);
 		gtk_container_set_border_width (GTK_CONTAINER (totem->plugins), 5);
-		gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (totem->plugins)->vbox), 2);
+		gtk_box_set_spacing (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (totem->plugins))), 2);
 		gtk_dialog_set_has_separator (GTK_DIALOG (totem->plugins), FALSE);
 
 		g_signal_connect_object (G_OBJECT (totem->plugins),
@@ -1244,7 +1393,7 @@ plugins_action_callback (GtkAction *action, Totem *totem)
 
 		manager = totem_plugin_manager_new ();
 		gtk_widget_show_all (GTK_WIDGET (manager));
-		gtk_container_add (GTK_CONTAINER (GTK_DIALOG (totem->plugins)->vbox),
+		gtk_container_add (GTK_CONTAINER (gtk_dialog_get_content_area (GTK_DIALOG (totem->plugins))),
 				   manager);
 	}
 
@@ -1314,31 +1463,55 @@ clear_playlist_action_callback (GtkAction *action, Totem *totem)
 	totem_action_set_mrl (totem, NULL, NULL);
 }
 
-static const GtkActionEntry seek_entries_ltr[] = {
-	{ "skip-forward", GTK_STOCK_MEDIA_FORWARD, N_("Skip _Forward"), "Right", N_("Skip forward"), G_CALLBACK (skip_forward_action_callback) },
-	{ "skip-backwards", GTK_STOCK_MEDIA_REWIND, N_("Skip _Backwards"), "Left", N_("Skip backwards"), G_CALLBACK (skip_backwards_action_callback) }
-};
+/* Show help in status bar when selecting (hovering over) a menu item. */
+static void
+menu_item_select_cb (GtkMenuItem *proxy, Totem *totem)
+{
+	GtkAction *action;
+	const gchar *message;
 
-static const GtkActionEntry seek_entries_rtl[] = {
-	{ "skip-forward", GTK_STOCK_MEDIA_FORWARD, N_("Skip _Forward"), "Left", N_("Skip forward"), G_CALLBACK (skip_forward_action_callback) },
-	{ "skip-backwards", GTK_STOCK_MEDIA_REWIND, N_("Skip _Backwards"), "Right", N_("Skip backwards"), G_CALLBACK (skip_backwards_action_callback) }
-};
+	action = gtk_activatable_get_related_action (GTK_ACTIVATABLE (proxy));
+	g_return_if_fail (action != NULL);
 
-static const GtkToggleActionEntry toggle_entries[] = {
-	{ "repeat-mode", NULL, N_("_Repeat Mode"), NULL, N_("Set the repeat mode"), G_CALLBACK (repeat_mode_action_callback), FALSE },
-	{ "shuffle-mode", NULL, N_("Shuff_le Mode"), NULL, N_("Set the shuffle mode"), G_CALLBACK (shuffle_mode_action_callback), FALSE },
-	{ "deinterlace", NULL, N_("_Deinterlace"), "I", N_("Deinterlace"), G_CALLBACK (deinterlace_action_callback), FALSE },
-	{ "show-controls", NULL, N_("Show _Controls"), "H", N_("Show controls"), G_CALLBACK (show_controls_action_callback), TRUE },
-	{ "sidebar", NULL, N_("S_idebar"), "F9", N_("Show or hide the sidebar"), G_CALLBACK (show_sidebar_action_callback), TRUE }
-};
+	message = gtk_action_get_tooltip (action);
+	if (message)
+		totem_statusbar_push_help (TOTEM_STATUSBAR (totem->statusbar), message);
+}
 
-static const GtkRadioActionEntry aspect_ratio_entries[] = {
-	{ "aspect-ratio-auto", NULL, N_("Auto"), NULL, N_("Sets automatic aspect ratio"), BVW_RATIO_AUTO },
-	{ "aspect-ratio-square", NULL, N_("Square"), NULL, N_("Sets square aspect ratio"), BVW_RATIO_SQUARE },
-	{ "aspect-ratio-fbt", NULL, N_("4:3 (TV)"), NULL, N_("Sets 4:3 (TV) aspect ratio"), BVW_RATIO_FOURBYTHREE },
-	{ "aspect-ratio-anamorphic", NULL, N_("16:9 (Widescreen)"), NULL, N_("Sets 16:9 (Anamorphic) aspect ratio"), BVW_RATIO_ANAMORPHIC },
-	{ "aspect-ratio-dvb", NULL, N_("2.11:1 (DVB)"), NULL, N_("Sets 2.11:1 (DVB) aspect ratio"), BVW_RATIO_DVB }
-};
+static void
+menu_item_deselect_cb (GtkMenuItem *proxy, Totem *totem)
+{
+	totem_statusbar_pop_help (TOTEM_STATUSBAR (totem->statusbar));
+}
+
+static void
+setup_action (Totem *totem, GtkAction *action)
+{
+	GSList *proxies;
+	for (proxies = gtk_action_get_proxies (action); proxies != NULL; proxies = proxies->next) {
+		if (GTK_IS_MENU_ITEM (proxies->data)) {
+			g_signal_connect (proxies->data, "select", G_CALLBACK (menu_item_select_cb), totem);
+			g_signal_connect (proxies->data, "deselect", G_CALLBACK (menu_item_deselect_cb), totem);
+		}
+
+	}
+}
+
+static void
+setup_menu_items (Totem *totem)
+{
+	GList *action_groups;
+
+	/* FIXME: We can remove this once GTK+ bug #574001 is fixed */
+	for (action_groups = gtk_ui_manager_get_action_groups (totem->ui_manager);
+	     action_groups != NULL; action_groups = action_groups->next) {
+		GtkActionGroup *action_group = GTK_ACTION_GROUP (action_groups->data);
+		GList *actions;
+		for (actions = gtk_action_group_list_actions (action_group); actions != NULL; actions = actions->next) {
+			setup_action (totem, GTK_ACTION (actions->data));
+		}
+	}
+}
 
 void
 totem_ui_manager_setup (Totem *totem)
@@ -1348,19 +1521,20 @@ totem_ui_manager_setup (Totem *totem)
 
 	/* FIXME: Moving these to GtkBuilder depends on bug #457631 */
 	if (gtk_widget_get_direction (totem->win) == GTK_TEXT_DIR_RTL) {
-		GList *actions = NULL;
 		GtkActionGroup *action_group = GTK_ACTION_GROUP (gtk_builder_get_object (totem->xml, "skip-action-group"));
+		GtkAction *action;
 
-		for (actions = gtk_action_group_list_actions (action_group); actions != NULL; actions = actions->next)
-			gtk_action_group_remove_action (action_group, GTK_ACTION (actions->data));
+		action = gtk_action_group_get_action (action_group, "skip-forward");
+		gtk_action_set_accel_path (action, "Left");
 
-		gtk_action_group_add_actions (action_group,
-				seek_entries_rtl,
-				G_N_ELEMENTS (seek_entries_rtl), totem);
+		action = gtk_action_group_get_action (action_group, "skip-backwards");
+		gtk_action_set_accel_path (action, "Right");
 	}
 
 	totem->ui_manager = GTK_UI_MANAGER (gtk_builder_get_object (totem->xml, "totem-ui-manager"));
 
+	setup_menu_items (totem);
+
 	totem->devices_action_group = NULL;
 	totem->devices_ui_id = gtk_ui_manager_new_merge_id (totem->ui_manager);
 	totem->languages_action_group = NULL;
